@@ -10,18 +10,18 @@ import (
 // Returns a representation of the string split into styled tokens. Any regexp
 // matches are highlighted. A nil regexp means no highlighting.
 //
-// minRunesCount: at least this many runes will be included in the result. If 0,
-// do all runes. For BenchmarkRenderHugeLine() performance.
+// maxTokensCount: at most this many tokens will be included in the result. If
+// 0, do all runes. For BenchmarkRenderHugeLine() performance.
 func (line *Line) HighlightedTokens(
 	plainTextStyle twin.Style,
 	searchHitStyle twin.Style,
 	search search.Search,
 	lineIndex linemetadata.Index,
-	minRunesCount int,
+	maxTokensCount int,
 ) textstyles.StyledRunesWithTrailer {
 	matchRanges := search.GetMatchRanges(line.Plain(lineIndex))
 
-	fromString := textstyles.StyledRunesFromString(plainTextStyle, string(line.raw), &lineIndex, minRunesCount)
+	fromString := textstyles.StyledRunesFromString(plainTextStyle, string(line.raw), &lineIndex, maxTokensCount)
 	returnRunes := make([]textstyles.CellWithMetadata, 0, len(fromString.StyledRunes))
 	lastWasSearchHit := false
 	for _, token := range fromString.StyledRunes {
